@@ -1,6 +1,7 @@
 # sorted in order of increasing complexity
 
 import numpy as np
+from tinygrad.tensor import Tensor
 
 class Optimizer:
   def __init__(self, params):
@@ -9,11 +10,11 @@ class Optimizer:
 class SGD(Optimizer):
   def __init__(self, params, lr=0.001):
     super(SGD, self).__init__(params)
-    self.lr = lr
+    self.lr = Tensor([lr], gpu=params[0].gpu)
 
   def step(self):
     for t in self.params:
-      t.data -= self.lr * t.grad
+      t -= t.grad * self.lr
 
 class RMSprop(Optimizer):
   def __init__(self, params, lr=0.001, decay=0.9, eps=1e-8):
@@ -26,8 +27,8 @@ class RMSprop(Optimizer):
 
   def step(self):
     for i, t in enumerate(self.params):
-      self.v[i] = self.decay * self.v[i] + (1 - self.decay) * np.square(t.grad)
-      t.data -= self.lr / (np.sqrt(self.v[i]) + self.eps) * t.grad
+      self.v[i] = self.decay * self.v[i] + (1 - self.decay) * np.square(t.grad.data)
+      t.data -= self.lr / (np.sqrt(self.v[i]) + self.eps) * t.grad.data
 
 class Adam(Optimizer):
   def __init__(self, params, lr=0.001, b1=0.9, b2=0.999, eps=1e-8):
@@ -43,10 +44,11 @@ class Adam(Optimizer):
 
   def step(self):
     self.t += 1
+    a = self.lr * (
+      np.sqrt(1 - np.power(self.b2, self.t)) /
+      (1 - np.power(self.b1, self.t)))
     for i,t in enumerate(self.params):
-      self.m[i] = self.b1 * self.m[i] + (1 - self.b1) * t.grad
-      self.v[i] = self.b2 * self.v[i] + (1 - self.b2) * np.square(t.grad)
-      mhat = self.m[i] / (1. - self.b1**self.t)
-      vhat = self.v[i] / (1. - self.b2**self.t)
-      t.data -= self.lr * mhat / (np.sqrt(vhat) + self.eps)
+      self.m[i] = self.b1 * self.m[i] + (1 - self.b1) * t.grad.data
+      self.v[i] = self.b2 * self.v[i] + (1 - self.b2) * np.square(t.grad.data)
+      t.data -= a * self.m[i] / (np.sqrt(self.v[i]) + self.eps)
 
